from logging.config import fileConfig

from sqlalchemy import engine_from_config
from sqlalchemy import pool

from alembic import context
from pathlib import Path
import sys
import os

# Get the directory containing this file (migrations directory)
migrations_dir = os.path.dirname(os.path.abspath(__file__))
# Get the root project directory (parent of migrations directory)
project_root = os.path.dirname(migrations_dir)
# Add the project root to sys.path
sys.path.insert(0, project_root)

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)

# Import Base after adding project_root to sys.path
try:
    from app.db.base import Base
    target_metadata = Base.metadata
except ImportError as e:
    print(f"Import error: {e}")
    # Fallback to direct import if module structure is an issue
    sys.path.append(os.path.join(project_root, 'app'))
    from db.base import Base
    target_metadata = Base.metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.

# Ensure the DB directory exists
DB_DIR = Path("/app/storage/db")
DB_DIR.mkdir(parents=True, exist_ok=True)

# Override sqlalchemy.url from the INI file
DB_PATH = DB_DIR / "db.sqlite"
config.set_main_option('sqlalchemy.url', f"sqlite:///{DB_PATH}")

def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = config.get_main_option("sqlalchemy.url")
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    connectable = engine_from_config(
        config.get_section(config.config_ini_section),
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    with connectable.connect() as connection:
        context.configure(
            connection=connection, target_metadata=target_metadata
        )

        with context.begin_transaction():
            context.run_migrations()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()